import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;


public final class OchiaiProfiling {
	
	public static class OutputString {
		String loc;
		double value;
		int occ;
	}
	
	public static class customComparator implements Comparator<OutputString>{

		@Override
		public int compare(OutputString o1, OutputString o2) {
			double v1 = ((OutputString)o1).value;
			double v2 = ((OutputString)o2).value;
			int occ1 = ((OutputString)o1).occ;
			int occ2 = ((OutputString)o2).occ;
			
			if (Math.abs(v1-v2)<1e-10)
				return occ1-occ2;
			return Double.compare(v1, v2);
		}
		
	}
	
	public static HashMap<String,Integer> getCovfromFiles(File f) {
		HashMap<String,Integer> hm = new HashMap<String,Integer>();
		try {
			BufferedReader br = new BufferedReader(new FileReader(f));
			String line = br.readLine();
			while (line!=null) {
				String key = line.trim();
				if (hm.containsKey(key)) {
					int val = hm.get(key);
					hm.put(key, val+1);
				} else {
					hm.put(key, 1);
				}
				line = br.readLine();
			}

			br.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
//		for (int i=0;i<hm.size();i++) 
//			System.out.println(hm.get(i));
		
		return hm;
	}
	public static HashMap<String,Integer> failureMap;
	public static HashMap<String,Integer> failureCntMap;
	public static HashMap<String,Integer> passMap;
	public static ArrayList<String> failureall;
	public static ArrayList<String> passall;
	
	public static void main(String[] args) {
		if (args.length!=1) { 
			System.err.println("Please select a dir.");
			return;
		}
		String dir = args[0];
		File folder = new File(dir);
		if (!folder.exists()) {
			System.err.println("Folder does not exist.");
			return;
		}
			
		File[] listOfFiles = folder.listFiles();
		int totalfailnum = 0;
		failureMap = new HashMap<String,Integer>();
		failureCntMap = new HashMap<String,Integer>();
		
		passMap = new HashMap<String,Integer>();
		failureall = null;
		passall = null;
//		predicateMap = new HashMap<String,Integer>();
		for (int i=0;i<listOfFiles.length;i++) {
			if (listOfFiles[i].isFile()) {
				String fn = listOfFiles[i].getName();
				if (fn.endsWith(".fail")) {
					HashMap<String,Integer> covset = getCovfromFiles(listOfFiles[i]);
					totalfailnum++;
					boolean directadd = false;
					if (failureall==null) {
						failureall = new ArrayList<String>();
						directadd = true;
					}
					Iterator<String> itr = covset.keySet().iterator();
					while (itr.hasNext()) {
						String key = itr.next();
						if (failureMap.containsKey(key)) {
							int occ = failureMap.get(key);
							
							failureMap.put(key,occ+covset.get(key));						
							occ = failureCntMap.get(key);
							failureCntMap.put(key, occ+1);

							}
						else {
							failureMap.put(key, covset.get(key));
							failureCntMap.put(key, 1);
						}
						
						if (directadd) {
							failureall.add(key);
						} 

					}
					
					if (!directadd) {
						for (int k=0;k<failureall.size();k++) {
							String key = failureall.get(k);
							if (!covset.containsKey(key)) {
								failureall.remove(k);
								k--;
							}
						}
					}
					
//					if (!pcset.contains("(Eq 91 (Read w8 0 U0))"))
//						System.out.println(fn);
		//			else 
		//				checknum++;
						
				} else if (fn.endsWith(".pass")) {
					HashMap<String,Integer> covset = getCovfromFiles(listOfFiles[i]);
					boolean directadd = false;
					if (passall==null) {
						passall = new ArrayList<String>();
						directadd = true;
					}

					Iterator<String> itr = covset.keySet().iterator();
					while (itr.hasNext()) {
						String key = itr.next();
						if (passMap.containsKey(key)) {
							int occ = passMap.get(key);
							
							passMap.put(key,occ+covset.get(key));						

							}
						else {
							passMap.put(key, covset.get(key));
						}
						
						if (directadd) {
							passall.add(key);
						} 

					}
					
					if (!directadd) {
						for (int k=0;k<passall.size();k++) {
							String key = passall.get(k);
							if (!covset.containsKey(key)) {
								passall.remove(k);
								k--;
							}
						}
					}
				}
			}
		}
		
//		for (int i=0;i<failureall.size();i++) {
//			String key = failureall.get(i);
//			if (!passall.contains(key))
//				System.out.println(key);
//		}
//		System.out.println(failureall.size());
		
		Object[] fails = failureMap.keySet().toArray();
		ArrayList<OutputString> outres = new ArrayList<OutputString>();
		for (int i=0;i<fails.length;i++) {
			int failureno = failureMap.get(fails[i]);
			int passno = 0;
			if (passMap.containsKey(fails[i])) 
				passno = passMap.get(fails[i]);
			double sus = (double)failureno/Math.sqrt(((double)(failureno+totalfailnum-failureCntMap.get(fails[i])))*((double)(failureno+passno)));
					
			OutputString newos = new OutputString();
			newos.loc = (String)fails[i];
			newos.value = sus;
			newos.occ = passno;
			outres.add(newos);
		}
		Collections.sort(outres, new customComparator());
		for (int i=0;i<outres.size();i++) {
			System.out.println(outres.get(i).loc+"*"+outres.get(i).value+"*"+outres.get(i).occ);
		}

	}
}
